@@ -28,9 +28,11 @@ import locale
 import pango
 import os
 import re
+import time
 from xml.sax import saxutils
 from series import SeriesManager, Show, Episode
 from lib import constants
+from lib.connectionmanager import ConnectionManager
 from lib.portrait import FremantleRotation
 from lib.util import get_color
 from settings import Settings
@@ -64,9 +66,14 @@ class MainWindow(hildon.StackableWindow):
  	self._rotation_manager = FremantleRotation(constants.SF_COMPACT_NAME,
                                                   self)
 
+        self.connection_manager = ConnectionManager()
+        self.connection_manager.connect('connection-changed',
+                                        self._on_connection_changed)
+
         self.series_manager = SeriesManager()
         self.settings = Settings()
         hildon.hildon_gtk_window_set_progress_indicator(self, True)
+        save_pid = AsyncItem(self.save_current_pid,())
         load_conf_item = AsyncItem(self.settings.load,
                                    (constants.SF_CONF_FILE,))
         load_shows_item = AsyncItem(self.series_manager.load,
@@ -84,9 +91,19 @@ class MainWindow(hildon.StackableWindow):
                                     self._update_show_art)
 
         self.request = AsyncWorker()
+        self.request.queue.put((0, save_pid))
         self.request.queue.put((0, load_conf_item))
         self.request.queue.put((0, load_shows_item))
-        self.request.start()
+
+        old_pid = self.get_previous_pid()
+
+        if old_pid:
+            show_information(self, _("Waiting for previous SeriesFinale to finish...."))
+            gobject.timeout_add(2000,
+                                self.run_after_pid_finish,
+                                old_pid, self.request.start)
+        else:
+            self.request.start()
 
         self.shows_view = ShowsSelectView()
         self.shows_view.connect('row-activated', self._row_activated_cb)
@@ -111,6 +128,31 @@ class MainWindow(hildon.StackableWindow):
 
         self._have_deleted = False
 
+    def save_current_pid(self):
+        pidfile = open(constants.SF_PID_FILE, 'w')
+        pidfile.write('%s' % os.getpid())
+        pidfile.close()
+
+    def get_previous_pid(self):
+        if not os.path.isfile(constants.SF_PID_FILE):
+            return None
+
+        pidfile = open(constants.SF_PID_FILE, 'r')
+        pid = pidfile.readline()
+        pidfile.close()
+        if os.path.exists('/proc/%s' % pid):
+            return pid
+        else:
+            os.remove(constants.SF_PID_FILE)
+            return None
+    
+    def run_after_pid_finish(self, pid, callback):
+        if os.path.exists('/proc/%s' % pid):
+            return True
+
+        callback()
+        return False
+
     def _load_finished(self, dummy_arg, error):
         self.shows_view.set_shows(self.series_manager.series_list)
         hildon.hildon_gtk_window_set_progress_indicator(self, False)
@@ -121,6 +163,7 @@ class MainWindow(hildon.StackableWindow):
                                  self.settings.getConf(Settings.SHOWS_SORT) != \
                                  Settings.RECENT_EPISODE)
         self._applyRotation()
+        self.series_manager.auto_save(True)
 
     def _create_menu(self):
         menu = hildon.AppMenu()
@@ -218,7 +261,7 @@ class MainWindow(hildon.StackableWindow):
 
     def _row_activated_cb(self, view, path, column):
         show = self.shows_view.get_show_from_path(path)
-        seasons_view = SeasonsView(self.settings, self.series_manager, show)
+        seasons_view = SeasonsView(self.settings, self.series_manager, self.connection_manager, show)
         seasons_view.connect('delete-event',
                      lambda w, e:
                         self.shows_view.update(show))
@@ -250,6 +293,8 @@ class MainWindow(hildon.StackableWindow):
         if not self.series_manager.series_list and not self._have_deleted:
             gtk.main_quit()
             return
+        self.series_manager.auto_save(False)
+
         save_shows_item = AsyncItem(self.series_manager.save,
                                (constants.SF_DB_FILE,))
         save_conf_item = AsyncItem(self.settings.save,
@@ -275,7 +320,7 @@ class MainWindow(hildon.StackableWindow):
             self.update_all_menu.hide()
         else:
             self.delete_menu.show()
-            self.update_all_menu.show()
+            self._on_connection_changed(self.connection_manager)
 
     def _update_all_shows_cb(self, button):
         hildon.hildon_gtk_window_set_progress_indicator(self, True)
@@ -334,6 +379,12 @@ class MainWindow(hildon.StackableWindow):
             return
         self.live_search.show()
 
+    def _on_connection_changed(self, connection_manager):
+        if connection_manager.is_online():
+            self.update_all_menu.show()
+        else:
+            self.update_all_menu.hide()
+
 class DeleteView(hildon.StackableWindow):
 
     def __init__(self,
@@ -399,7 +450,7 @@ class ShowsSelectView(gtk.TreeView):
         self.tree_model.add_shows(shows)
 
     def get_show_from_path(self, path):
-        return self.tree_model[path][self.tree_model.SHOW_COLUMN]
+        return self.get_model()[path][ShowListStore.SHOW_COLUMN]
 
     def sort_by_recent_date(self):
         self.tree_model.set_sort_column_id(self.tree_model.NEXT_EPISODE_COLUMN,
@@ -464,6 +515,7 @@ class ShowListStore(gtk.ListStore):
         self.set_value(iter, self.INFO_COLUMN, info_markup)
         self.set_value(iter, self.NEXT_EPISODE_COLUMN,
                        info['next_episode'])
+        self.set_value(iter, self.SEARCH_COLUMN, show.name.lower())
         if pixbuf_is_cover(pixbuf):
             return
         if show.image and os.path.isfile(show.image):
@@ -503,7 +555,7 @@ class ShowListStore(gtk.ListStore):
 
 class SeasonsView(hildon.StackableWindow):
 
-    def __init__(self, settings, series_manager, show):
+    def __init__(self, settings, series_manager, connection_manager, show):
         super(SeasonsView, self).__init__()
         self.set_title(show.name)
 
@@ -515,6 +567,9 @@ class SeasonsView(hildon.StackableWindow):
         self.series_manager.connect('updated-show-art',
                                     self._update_show_art)
 
+        self.connection_manager = connection_manager
+        self.connection_manager.connect('connection-changed',
+                                        self._on_connection_changed)
         self.show = show
         self.set_app_menu(self._create_menu())
         self.set_title(show.name)
@@ -590,6 +645,7 @@ class SeasonsView(hildon.StackableWindow):
             self.update_menu.hide()
         else:
             self.update_menu.show()
+            self._on_connection_changed(self.connection_manager)
 
     def _update_series_cb(self, button):
         self.request = self.series_manager.update_show_episodes(self.show)
@@ -682,6 +738,12 @@ class SeasonsView(hildon.StackableWindow):
         seasons = self.show.get_seasons()
         seasons_delete_view.show_all()
 
+    def _on_connection_changed(self, connection_manager):
+        if connection_manager.is_online():
+            self.update_menu.show()
+        else:
+            self.update_menu.hide()
+
 class SeasonsDeleteView(DeleteView):
 
     def __init__(self, series_manager, seasons_select_view):
@@ -1108,7 +1170,7 @@ class EpisodeListStore(gtk.ListStore):
         for episode in episode_list:
             name = str(episode)
             row = {self.CHECK_COLUMN: episode.watched,
-                   self.INFO_COLUMN: name,
+                   self.INFO_COLUMN: saxutils.escape(str(name)),
                    self.EPISODE_COLUMN: episode}
             self.append(row.values())
         self.update()
@@ -1132,9 +1194,8 @@ class EpisodesCheckView(gtk.TreeView):
         model = EpisodeListStore()
         episode_renderer = gtk.CellRendererText()
         episode_renderer.set_property('ellipsize', pango.ELLIPSIZE_END)
-        if model.CHECK_COLUMN != 0:
-            episode_renderer.set_property('width', gtk.gdk.screen_get_default().get_width() - 150)
         column = gtk.TreeViewColumn('Name', episode_renderer, markup = model.INFO_COLUMN)
+        column.set_property('expand', True)
         self.append_column(column)
         self.watched_renderer = gtk.CellRendererToggle()
         self.watched_renderer.set_property('width', 100)
@@ -1195,6 +1256,8 @@ class EpisodeView(hildon.StackableWindow):
         super(EpisodeView, self).__init__()
         self.episode = episode
 
+        self.set_app_menu(self._create_menu())
+
         contents_area = hildon.PannableArea()
         contents_area.connect('horizontal-movement',
                               self._horizontal_movement_cb)
@@ -1205,14 +1268,12 @@ class EpisodeView(hildon.StackableWindow):
         self._update_info_text_view()
         contents.add(self.infotextview)
 
-        self.set_app_menu(self._create_menu())
-
         self.add(contents_area)
 
     def _update_info_text_view(self):
         self.infotextview.clear()
-        self.infotextview.set_title('%(number)s - %(name)s' % {'name': self.episode.name,
-                                                               'number': self.episode.get_episode_show_number()})
+        self._set_episode_title()
+        self.check_button.set_active(self.episode.watched)
         self.infotextview.add_field(self.episode.overview)
         self.infotextview.add_field('\n')
         self.infotextview.add_field(self.episode.get_air_date_text(),
@@ -1223,6 +1284,11 @@ class EpisodeView(hildon.StackableWindow):
         self.infotextview.add_field(self.episode.rating, _('Rating'))
         self.set_title(self.episode.name)
 
+    def _set_episode_title(self):
+        self.infotextview.set_title('%(number)s - %(name)s' % {'name': self.episode.name,
+                                                               'number': self.episode.get_episode_show_number()},
+                                    self.episode.watched)
+
     def _create_menu(self):
         menu = hildon.AppMenu()
 
@@ -1231,6 +1297,12 @@ class EpisodeView(hildon.StackableWindow):
         button.connect('clicked', self._edit_episode_cb)
         menu.append(button)
 
+        self.check_button = hildon.CheckButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self.check_button.set_label(_('Watched'))
+        self.check_button.set_active(self.episode.watched)
+        self.check_button.connect('toggled', self._watched_button_toggled_cb)
+        menu.append(self.check_button)
+
         menu.show_all()
         return menu
 
@@ -1263,6 +1335,10 @@ class EpisodeView(hildon.StackableWindow):
             self.episode = episode
             self._update_info_text_view()
 
+    def _watched_button_toggled_cb(self, button):
+        self.episode.watched = button.get_active()
+        self._set_episode_title()
+
 class EpisodesDeleteView(DeleteView):
 
     def __init__(self, show):
@@ -1309,27 +1385,39 @@ class EpisodesSelectView(gtk.TreeView):
 
 class InfoTextView(hildon.TextView):
 
+    TEXT_TAG = 'title'
+
     def __init__(self):
         super(InfoTextView, self).__init__()
 
         buffer = gtk.TextBuffer()
-        self.iter = buffer.get_start_iter()
 
         self.set_buffer(buffer)
         self.set_wrap_mode(gtk.WRAP_WORD)
         self.set_editable(False)
         self.set_cursor_visible(False)
 
-    def set_title(self, title):
+    def set_title(self, title, strike = False):
         if not title:
             return
-        title_tag = gtk.TextTag()
+        text_buffer = self.get_buffer()
+        tag_table = text_buffer.get_tag_table()
+        title_tag = tag_table.lookup(self.TEXT_TAG)
+        if not title_tag:
+            title_tag = gtk.TextTag(self.TEXT_TAG)
+            tag_table.add(title_tag)
+        else:
+            title_end_iter = text_buffer.get_start_iter()
+            if not title_end_iter.forward_to_tag_toggle(title_tag):
+                title_end_iter = text_buffer.get_start_iter()
+            text_buffer.delete(text_buffer.get_start_iter(),
+                               title_end_iter)
+
         title_tag.set_property('weight', pango.WEIGHT_BOLD)
         title_tag.set_property('size', pango.units_from_double(24.0))
         title_tag.set_property('underline-set', True)
-        tag_table = self.get_buffer().get_tag_table()
-        tag_table.add(title_tag)
-        self.get_buffer().insert_with_tags(self.iter, str(title) + '\n', title_tag)
+        title_tag.set_property('strikethrough', strike)
+        text_buffer.insert_with_tags(text_buffer.get_start_iter(), str(title) + '\n', title_tag)
 
     def add_field(self, contents, label = None):
         if not contents:
@@ -1338,14 +1426,13 @@ class InfoTextView(hildon.TextView):
             contents = _('\n%(label)s: %(contents)s') % {'label': label,
                                                          'contents': contents,
                                                         }
-        self.get_buffer().insert(self.iter, contents)
+        self.get_buffer().insert(self.get_buffer().get_end_iter(), contents)
 
     def clear(self):
         buffer = self.get_buffer()
         if not buffer:
             return
         buffer.delete(buffer.get_start_iter(), buffer.get_end_iter())
-        self.iter = buffer.get_start_iter()
 
 class LiveSearchEntry(gtk.HBox):
 
@@ -1421,6 +1508,9 @@ class NewShowsDialog(gtk.Dialog):
         contents.add(self.manual_add_button)
         self.vbox.add(contents)
         self.vbox.show_all()
+        parent.connection_manager.connect('connection-changed',
+                                          self._on_connection_changed)
+        self._on_connection_changed(parent.connection_manager)
 
     def _button_clicked_cb(self, button):
         if button == self.search_shows_button:
@@ -1428,6 +1518,11 @@ class NewShowsDialog(gtk.Dialog):
         elif button == self.manual_add_button:
             self.response(self.ADD_MANUALLY_RESPONSE)
 
+    def _on_connection_changed(self, connection_manager):
+        if connection_manager.is_online():
+            self.search_shows_button.show()
+        else:
+            self.search_shows_button.hide()
 
 class FoundShowListStore(gtk.ListStore):
 
